{*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  Lazarus GDALDataset Shadow Class.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}
unit gdaldataset;

{$IFDEF FPC}
{$mode objfpc}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, gdal, gdalcore, gdalrasterband;

type

  TGDALDataset = class(TGDALMajorObject)
  private
    //FObj : TGDALRasterBandH;
    FOwned : boolean;
    FXSize : longint;
    FYSize : longint;
    FBandCount : longint;
    FProjectRef : PCHAR;
    FWKT : PCHAR;
  public
    property  Owned : boolean read FOwned write FOwned;
    property  XSize : longint read FXSize write FXSize;
    property  YSize : longint read FYSize write FYSize;
    property  BandCount : longint read FBandCount write FBandCount;
    property  ProjectRef : PCHAR read FProjectRef write FProjectRef;

    function  Open (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH;
    function  OpenShared (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH;
    function  GetRasterBandXSize : longint;
    function  GetRasterBandYSize : longint;
    function  GetRasterCount : longint;
    function  GetRasterBand (const BandNo : longint) : TGDALRasterBandH;
    procedure FlushCache;
    function  GetProjectionRef : PCHAR;
    function  SetProjection (const WKTProj : PCHAR) : TCPLErr;
    function  GetGeoTransform (Geotransform : PDOUBLE) : TCPLErr;
    function  SetGeoTransform (Geotransform : PDOUBLE) : TCPLErr;
    function  AddBand(eType : TGDALDataType; papszOptions : PPCHAR) : TCPLErr;
    procedure GetInternalHandle(hDS : TGDALDatasetH; const pszRequest : PCHAR);
    function  GetDriver : TGDALDriverH;
    function  GetFileList : PPCHAR;
    function  GetGCPCount : longint;
    function  GetGCPProjection : PCHAR;
    function  GetGCPs : PGDAL_GCP;
    function  SetGCPs(nGCPCount : longint;
                      const pasGCPList : PGDAL_GCP;
                      const pszGCPProjection : PCHAR) : TCPLErr;
    function  AdviseRead(nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize : longint;
                         eDT : TGDALDataType; nBandCount : longint;
                         panBandMap : PINTEGER; papszOptions : PPCHAR
                        ) : TCPLErr;
    function  RasterIO(eRWFlag : TGDALRWFlag;
                       nXOff, YOff, nXSize: longint;
                       pData : POINTER;
                       nBufXSize, nBufYSize : longint;
                       eBufType : TGDALDataType;
                       nBandCount : longint;
                       panBandMap : PINTEGER;
                       nPixelSpace, nLineSpace, nBandSpace : longint
                      ) : TCPLErr;
    function  ReferenceDataset : longint;
    function  DereferenceDataset : longint;
    function  BuildOverviews(const pszResampling : PCHAR;
          	             nOverviews : longint;
          	             panOverviewList : PINTEGER;
          	             nListBands : longint;
          	             panBandList : PINTEGER;
          	             pfnProgress : TGDALProgressFunc;
          	             pProgressData : POINTER) : TCPLErr;
    procedure CInit(const obja : TGDALDatasetH; const ownedA : boolean);
    procedure CloseDS;
    function  IsValid : Boolean;
    function  GetObjPtr : TGDALDatasetH;
    constructor Create;
    destructor Destroy; override;
  end;

implementation
constructor TGDALDataset.Create;
begin
  inherited create;
  Handle := NIL;
  FOwned := false;
  FXSize := 0;
  FYSize := 0;
  FBandCount := 0;
end;

destructor TGDALDataset.Destroy;
begin
  if (Handle <> NIL) and (FOwned = true) then
    CloseDS;
  inherited Destroy;
end;

//' ----------------------------------------------------------------------------
procedure TGDalDataSet.CInit(const objA : TGDALDatasetH; const ownedA : boolean );
begin
  Handle := objA;
  Owned := ownedA;
    if (Handle <> NIL) then
    begin
      FXSize := gdalcore.GDALGetRasterXSize(Handle);
      FYSize := gdalcore.GDALGetRasterYSize(Handle);
      FBandCount := gdalcore.GDALGetRasterCount(Handle);
    end;
end;

//' ----------------------------------------------------------------------------
function TGDALDataset.IsValid : Boolean;
begin
  if (Handle = NIL) then
      result := false
  else
      result := true;
end;

//' ----------------------------------------------------------------------------
function TGDALDataset.GetObjPtr : TGDALDatasetH;
begin
    result := Handle;
end;

function  TGDALDataset.Open (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH;
begin
  result := GDALCore.GDALOpen(Filename, Access);
end;

function  TGDALDataset.OpenShared (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH;
begin
  result := GDALCore.GDALOpenShared(Filename, Access);
end;

function  TGDALDataset.GetRasterBandXSize : longint;
begin
  if (Handle <> NIL) then
  begin
    FXSize := GDALCore.GDALGetRasterBandXSize(Handle);
    result := FXSize;
  end;
end;

function  TGDALDataset.GetRasterBandYSize : longint;
begin
  if (Handle <> NIL) then
  begin
    FYSize := GDALCore.GDALGetRasterBandYSize(Handle);
    result := FYSize;
  end;
end;

function  TGDALDataset.GetRasterCount : longint;
begin
  if (Handle <> NIL) then
  begin
    FBandCount := GDALCore.GDALGetRasterCount(Handle);
    result := FBandCount;
  end;
end;

function  TGDALDataset.GetRasterBand (const BandNo : longint) : TGDALRasterBandH;
begin
  if (Handle <> NIL) then
    result := GDALCore.GDALGetRasterBand(Handle, BandNo);
end;

procedure TGDALDataset.FlushCache;
begin
  if (Handle <> NIL) then
    GDALCore.GDALFlushCache(Handle);
end;

function  TGDALDataset.GetProjectionRef : PCHAR;
begin
  if (Handle <> NIL) then
    result := GDALCore.GDALGetProjectionRef(Handle)
  else
    result := NIL;
end;

function  TGDALDataset.SetProjection (const WKTProj : PCHAR) : TCPLErr;
begin
  if (Handle <> NIL) then
  begin
    result := GDALCore.GDALSetProjection(Handle, WKTProj);
  end;
end;

function  TGDALDataset.GetDriver : TGDALDriverH;
begin
  result := GDALCore.GDALGetDatasetDriver(Handle);
end;

//' ----------------------------------------------------------------------------
procedure TGDALDataset.CloseDS;
begin
  if (Handle <> NIL) then
  begin
    GDALCore.GDALClose(Handle);
    Handle := NIL;
    FOwned := false;
  end;
end;

//' ----------------------------------------------------------------------------
{function TGDALDataset.GetRasterBand(const BandNo : longint) : TGDALRasterBand;
var
  c_obj : TGDALRasterBandH;
  band : TGDALRasterBand;
begin
  band := NIL;
  if (FObj <> NIL) then
  begin
    c_obj := GDALCore.GDALGetRasterBand(FObj, BandNo);
    if (c_obj <> NIL) then
    begin
      band := TGDALRasterBand.Create;
      band.CInit(c_obj);
    end;
  end;
  result := band;
end;
}
//' ----------------------------------------------------------------------------
function TGDALDataset.GetGeoTransform(Geotransform : PDOUBLE) : TCPLErr;
var
  gtsafe : TDoubleArray6;
begin
  if (Handle <> NIL) then
  begin
{$IFDEF FPC}
    result := GDALCore.GDALGetGeoTransform(Handle, gtsafe);
    Geotransform[0] := gtsafe[0];
    Geotransform[1] := gtsafe[1];
    Geotransform[2] := gtsafe[2];
    Geotransform[3] := gtsafe[3];
    Geotransform[4] := gtsafe[4];
    Geotransform[5] := gtsafe[5];
{$ELSE}
    result := GDALCore.GDALGetGeoTransform(Handle, Geotransform);
{$ENDIF}
  end
  else
    result := CE_None;
end;

//' ----------------------------------------------------------------------------
function TGDALDataset.SetGeoTransform(Geotransform : PDOUBLE) : TCPLErr;
var
  gtsafe : TDoubleArray6;
  i : integer;
begin
  if (Handle <> NIL) then
  begin
  {$IFDEF FPC}
    for i := 0 To 5 do
      gtsafe[i] := Geotransform[i];
    result := GDALCore.GDALSetGeoTransform(Handle, @gtsafe);
  {$ELSE}
    Result := GDALCore.GDALSetGeoTransform(Handle, Geotransform);
  {$ENDIF}
  end;
end;

function  TGDALDataset.AddBand(eType : TGDALDataType; papszOptions : PPCHAR) : TCPLErr;
begin
  if (Handle <> NIL) then
    result := GDALCore.GDALAddBand(Handle, eType, papszOptions);
end;

procedure TGDALDataset.GetInternalHandle(hDS : TGDALDatasetH; const pszRequest : PCHAR);
begin
  if (Handle <> NIL) then
    GDALCore.GDALGetInternalHandle(Handle, pszRequest);
end;

function TGDALDataset.GetFileList : PPCHAR;
begin
  result := GDALCore.GDALGetFileList(Handle);
end;

function  TGDALDataset.GetGCPCount : longint;
begin
  result := GDALCore.GDALGetGCPCount(Handle);
end;

function  TGDALDataset.GetGCPProjection : PCHAR;
begin
  result := GDALCore.GDALGetGCPProjection(Handle);
end;

function  TGDALDataset.GetGCPs : PGDAL_GCP;
begin
  result := GDALCore.GDALGetGCPs(Handle);
end;

function  TGDALDataset.SetGCPs(nGCPCount : longint; const pasGCPList : PGDAL_GCP;
                               const pszGCPProjection : PCHAR) : TCPLErr;
begin
  result := GDALCore.GDALSetGCPs(Handle, nGCPCount, pasGCPList, pszGCPProjection);
end;

function  TGDALDataset.AdviseRead(nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize : longint;
                                         eDT : TGDALDataType; nBandCount : longint;
                                         panBandMap : PINTEGER; papszOptions : PPCHAR
                                         ) : TCPLErr;
begin
  result := GDALCore.GDALDatasetAdviseRead(Handle, nXOff, nYOff, nXSize, nYSize,
                                           nBufXSize, nBufYSize, eDT, nBandCount,
                                           panBandMap, papszOptions);
end;

function  TGDALDataset.RasterIO(eRWFlag : TGDALRWFlag;
                                nXOff, YOff, nXSize: longint;
                                pData : POINTER;
                                nBufXSize, nBufYSize : longint;
                                eBufType : TGDALDataType;
                                nBandCount : longint;
                                panBandMap : PINTEGER;
                                nPixelSpace, nLineSpace, nBandSpace : longint
                               ) : TCPLErr;
begin
  result := GDALCore.GDALDatasetRasterIO(Handle, eRWFlag, nXOff, YOff, nXSize,
                                         pData, nBufXSize, nBufYSize, eBufType,
                                         nBandCount, panBandMap, nPixelSpace,
                                         nLineSpace, nBandSpace);
end;

function  TGDALDataset.ReferenceDataset : longint;
begin
  result := GDALCore.GDALReferenceDataset(Handle);
end;

function  TGDALDataset.DereferenceDataset : longint;
begin
  result := GDALCore.GDALDereferenceDataset(Handle);
end;

function  TGDALDataset.BuildOverviews(const pszResampling : PCHAR;
                                      nOverviews : longint;
                                      panOverviewList : PINTEGER;
                                      nListBands : longint;
                                      panBandList : PINTEGER;
                                      pfnProgress : TGDALProgressFunc;
                                      pProgressData : POINTER) : TCPLErr;
begin
  result := GDALCore.GDALBuildOverviews(Handle, pszResampling, nOverviews,
                                      panOverviewList, nListBands, panBandList,
                                      pfnProgress, pProgressData);
end;


end.

